#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Copyright (c) 2023-2024, AllWorldIT
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Smart SSH Client configuration."""

import os.path
import pathlib
import sys
import tomllib as toml
from typing import Dict, List, Optional

import gnupg

__all__ = ["get_host_config"]


CONFIG_HOSTS_DIR = "~/.config/smart-ssh-client/hosts.d"


def find_config_files_walk(config_directory: str) -> List[str]:
    """Find all config files in a directory."""
    target_extensions = (".ini", ".ini.gpg")
    found_files: List[str] = []

    for root, _, files in os.walk(config_directory, followlinks=True):
        for file in files:
            if file.endswith(target_extensions):
                found_files.append(os.path.join(root, file))

    return sorted(found_files)


def find_host_config(config_file: str, hostname: str, port: Optional[str]) -> dict[str, str]:
    """Get host configuration."""

    host_config: Dict[str, str] = {}

    # If the file doesn't exist, just return an empty dict
    if not os.path.exists(os.path.expanduser(config_file)):
        return host_config

    decrypted_data = None
    # Try encrypted config file
    if config_file.endswith(".ini.gpg"):
        # Initialize GnuPG
        gpg = gnupg.GPG()
        # Open encrypted hosts file
        with open(os.path.expanduser(config_file), "rb") as encrypted_file:
            encrypted_data = encrypted_file.read()
        # Decrypt the data in memory
        decrypted_data = gpg.decrypt(encrypted_data).data.decode("UTF-8")
    # Try unencrypted config file
    elif config_file.endswith(".ini"):
        with open(os.path.expanduser(config_file), "r", encoding="UTF-8") as hosts_file:
            decrypted_data = hosts_file.read()
    # Check if we have decrypted data, if we do, turn it into an ini config
    if decrypted_data:
        # Parse decrypted ini file
        config = toml.loads(decrypted_data)
        # Check if we have a host defined
        host_config_name = None
        # We first check by hostname:port
        if port and config.get(f"{hostname}:{port}"):
            host_config_name = f"{hostname}:{port}"
        # Then just by hostname
        if config.get(hostname):
            host_config_name = hostname
        # If we found a host key, return the config
        if host_config_name:
            # Try work out a brief config file name
            config_file_components = config_file.split("/")
            if len(config_file_components) > 6:
                brief_config_file = str(pathlib.Path(*config_file_components[6:]))
            else:
                brief_config_file = config_file
            print(
                f"PROXY INFO: Smart SSH Client - Loaded host '{host_config_name}' configuration from '{brief_config_file}'",
                file=sys.stderr,
            )
            # Set host config
            host_config = config[host_config_name]

    return host_config


def get_host_config(hostname: str, port: Optional[str]) -> dict[str, str]:
    """Get host configuration."""

    host_config = {}

    # Grab a list of config files
    config_file_list = find_config_files_walk(os.path.expanduser(CONFIG_HOSTS_DIR))

    # Loop with all config files
    for config_file in config_file_list:
        host_config = find_host_config(config_file, hostname, port)
        if host_config:
            break

    return host_config
